import _defineProperty from 'babel-runtime/helpers/defineProperty';
import _extends from 'babel-runtime/helpers/extends';
import _objectWithoutProperties from 'babel-runtime/helpers/objectWithoutProperties';
import _regeneratorRuntime from 'babel-runtime/regenerator';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import * as tslib_1 from "tslib";
import React, { cloneElement, Component, isValidElement } from 'react';
import PropTypes from 'prop-types';
import { observer } from 'mobx-react';
import { computed, isArrayLike } from 'mobx';
import classNames from 'classnames';
import omit from 'lodash/omit';
import isString from 'lodash/isString';
import { pxToRem } from '../../../es/_util/UnitConvertor';
import KeyCode from '../../../es/_util/KeyCode';
import measureScrollbar from '../../../es/_util/measureScrollbar';
import Record from '../data-set/Record';
import TableContext from './TableContext';
import { findCell, findFirstFocusableElement, getAlignByField, getColumnKey, getEditorByColumnAndRecord, isDisabledRow, isRadio } from './utils';
import { TableCommandType } from './enum';
import CheckBox from '../check-box/CheckBox';
import Output from '../output/Output';
import { default as Button } from '../button/Button';
import { $l } from '../locale-context';
import Tooltip from '../tooltip/Tooltip';
import autobind from '../_util/autobind';
var inTab = false;
var TableCell = function (_Component) {
    _inherits(TableCell, _Component);

    function TableCell() {
        _classCallCheck(this, TableCell);

        return _possibleConstructorReturn(this, (TableCell.__proto__ || Object.getPrototypeOf(TableCell)).apply(this, arguments));
    }

    _createClass(TableCell, [{
        key: 'handleEditorKeyDown',
        value: function handleEditorKeyDown(e) {
            switch (e.keyCode) {
                case KeyCode.TAB:
                    var _props = this.props,
                        prefixCls = _props.prefixCls,
                        column = _props.column;

                    var cell = findCell(this.context.tableStore, prefixCls, getColumnKey(column));
                    var node = findFirstFocusableElement(cell);
                    if (node) {
                        inTab = true;
                        node.focus();
                    }
                    break;
                default:
            }
        }
    }, {
        key: 'handleFocus',
        value: function handleFocus(e) {
            var tableStore = this.context.tableStore;
            var currentEditorName = tableStore.currentEditorName,
                dataSet = tableStore.dataSet,
                inlineEdit = tableStore.inlineEdit;
            var _props2 = this.props,
                prefixCls = _props2.prefixCls,
                record = _props2.record,
                column = _props2.column,
                lock = _props2.column.lock;

            if (!currentEditorName && !isDisabledRow(record) && (!inlineEdit || record.editing)) {
                dataSet.current = record;
                this.showEditor(e.currentTarget, lock);
                if (!this.cellEditor || isRadio(this.cellEditor)) {
                    var cell = findCell(tableStore, prefixCls, getColumnKey(column), lock);
                    var node = findFirstFocusableElement(cell);
                    if (node && !inTab) {
                        node.focus();
                    }
                }
            }
            inTab = false;
        }
    }, {
        key: 'handleCommandEdit',
        value: function handleCommandEdit() {
            var record = this.props.record;
            var tableStore = this.context.tableStore;

            if (tableStore.inlineEdit) {
                tableStore.currentEditRecord = record;
            }
        }
    }, {
        key: 'handleCommandDelete',
        value: function handleCommandDelete() {
            var record = this.props.record;
            var tableStore = this.context.tableStore;
            var dataSet = tableStore.dataSet;

            dataSet['delete'](record);
        }
    }, {
        key: 'handleCommandSave',
        value: function () {
            var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
                var tableStore, dataSet;
                return _regeneratorRuntime.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                tableStore = this.context.tableStore;
                                dataSet = tableStore.dataSet;
                                _context.next = 4;
                                return dataSet.submit();

                            case 4:
                                _context.t0 = _context.sent;

                                if (!(_context.t0 !== false)) {
                                    _context.next = 7;
                                    break;
                                }

                                tableStore.currentEditRecord = void 0;

                            case 7:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, this);
            }));

            function handleCommandSave() {
                return _ref.apply(this, arguments);
            }

            return handleCommandSave;
        }()
    }, {
        key: 'handleCommandCancel',
        value: function handleCommandCancel() {
            var record = this.props.record;
            var tableStore = this.context.tableStore;

            if (record.status === "add" /* add */) {
                    var dataSet = tableStore.dataSet;

                    dataSet.remove(record);
                } else {
                record.reset();
                tableStore.currentEditRecord = void 0;
            }
        }
    }, {
        key: 'getButtonProps',
        value: function getButtonProps(type, record) {
            var disabled = isDisabledRow(record);
            switch (type) {
                case TableCommandType.edit:
                    return { icon: 'mode_edit', onClick: this.handleCommandEdit, disabled: disabled, title: $l('Table', 'edit_button') };
                case TableCommandType['delete']:
                    return { icon: 'delete', onClick: this.handleCommandDelete, disabled: disabled, title: $l('Table', 'delete_button') };
                default:
            }
        }
    }, {
        key: 'renderCommand',
        value: function renderCommand() {
            var _this2 = this;

            var record = this.props.record;

            var command = this.getCommand();
            if (record.editing) {
                return [React.createElement(
                    Tooltip,
                    { key: 'save', title: $l('Table', 'save_button') },
                    React.createElement(Button, { color: "primary" /* primary */, funcType: "flat" /* flat */, icon: 'finished', onClick: this.handleCommandSave })
                ), React.createElement(
                    Tooltip,
                    { key: 'cancel', title: $l('Table', 'cancel_button') },
                    React.createElement(Button, { color: "primary" /* primary */, funcType: "flat" /* flat */, icon: 'cancle_a', onClick: this.handleCommandCancel })
                )];
            }
            if (command) {
                var children = [];
                command.forEach(function (button) {
                    var props = {};
                    if (isArrayLike(button)) {
                        props = button[1];
                        button = button[0];
                    }
                    if (isString(button) && button in TableCommandType) {
                        var defaultButtonProps = _this2.getButtonProps(button, record);
                        if (defaultButtonProps) {
                            var title = defaultButtonProps.title,
                                otherProps = _objectWithoutProperties(defaultButtonProps, ['title']);

                            children.push(React.createElement(
                                Tooltip,
                                { key: button, title: title },
                                React.createElement(Button, _extends({ color: "primary" /* primary */, funcType: "flat" /* flat */ }, otherProps, props))
                            ));
                        }
                    } else if (isValidElement(button)) {
                        children.push(button);
                    }
                });
                return children;
            }
        }
    }, {
        key: 'renderEditor',
        value: function renderEditor() {
            var cellEditor = this.cellEditor;

            if (isValidElement(cellEditor)) {
                var _context$tableStore = this.context.tableStore,
                    dataSet = _context$tableStore.dataSet,
                    pristine = _context$tableStore.pristine;
                var _props3 = this.props,
                    name = _props3.column.name,
                    record = _props3.record;
                var checkField = dataSet.props.checkField;

                var newEditorProps = _extends({}, cellEditor.props, {
                    record: record,
                    name: name,
                    pristine: pristine,
                    disabled: isDisabledRow(record),
                    indeterminate: checkField && checkField === name && record.isIndeterminate,
                    labelLayout: "none" /* none */
                });
                return cloneElement(cellEditor, newEditorProps);
            }
        }
    }, {
        key: 'getCheckBox',
        value: function getCheckBox() {
            var record = this.props.record;
            var dataSet = this.context.tableStore.dataSet;
            var checkField = dataSet.props.checkField;

            if (checkField) {
                return React.createElement(CheckBox, { name: checkField, record: record, disabled: isDisabledRow(record), indeterminate: record.isIndeterminate });
            }
        }
    }, {
        key: 'getCommand',
        value: function getCommand() {
            var _props4 = this.props,
                command = _props4.column.command,
                record = _props4.record;
            var dataSet = this.context.tableStore.dataSet;

            if (typeof command === 'function') {
                return command({ dataSet: dataSet, record: record });
            }
            return command;
        }
    }, {
        key: 'getCellRenderer',
        value: function getCellRenderer(command) {
            var column = this.props.column;
            var renderer = column.renderer;

            if (command) {
                return this.renderCommand;
            }
            if (this.cellEditorInCell) {
                return this.renderEditor;
            }
            return renderer;
        }
    }, {
        key: 'getInnerNode',
        value: function getInnerNode(prefixCls, command) {
            var _context$tableStore2 = this.context.tableStore,
                rowHeight = _context$tableStore2.rowHeight,
                expandIconAsCell = _context$tableStore2.expandIconAsCell,
                hasCheckFieldColumn = _context$tableStore2.hasCheckFieldColumn,
                pristine = _context$tableStore2.pristine,
                children = this.props.children;

            if (expandIconAsCell && children) {
                return children;
            }
            var _props5 = this.props,
                column = _props5.column,
                record = _props5.record,
                indentSize = _props5.indentSize;
            var name = column.name;
            var hasEditor = this.hasEditor;

            var innerProps = {
                className: prefixCls + '-inner',
                tabIndex: hasEditor && !isDisabledRow(record) ? 0 : -1,
                onFocus: this.handleFocus,
                pristine: pristine
            };
            if (!hasEditor) {
                innerProps.onKeyDown = this.handleEditorKeyDown;
            }
            if (rowHeight !== 'auto') {
                innerProps.style = {
                    height: pxToRem(rowHeight)
                };
            }
            var indentText = children && React.createElement('span', { style: { paddingLeft: pxToRem(indentSize * record.level) } });
            var checkBox = children && !hasCheckFieldColumn && this.getCheckBox();
            var prefix = (indentText || children || checkBox) && React.createElement(
                'span',
                { key: 'prefix', className: prefixCls + '-prefix', style: innerProps.style },
                indentText,
                children,
                checkBox
            );
            return [prefix, React.createElement(Output, _extends({ key: 'output' }, innerProps, { record: record, renderer: this.getCellRenderer(command), name: name, disabled: isDisabledRow(record), showHelp: "none" /* none */ }))];
        }
    }, {
        key: 'render',
        value: function render() {
            var _ref2;

            var _props6 = this.props,
                column = _props6.column,
                prefixCls = _props6.prefixCls,
                record = _props6.record;
            var _context$tableStore3 = this.context.tableStore,
                inlineEdit = _context$tableStore3.inlineEdit,
                pristine = _context$tableStore3.pristine;
            var className = column.className,
                style = column.style,
                align = column.align,
                name = column.name,
                onCell = column.onCell;

            var command = this.getCommand();
            var field = name ? record.getField(name) : void 0;
            var cellPrefix = prefixCls + '-cell';
            var cellExternalProps = typeof onCell === 'function' ? onCell({
                dataSet: record.dataSet,
                record: record,
                column: column
            }) : {};
            var cellStyle = _extends({
                textAlign: align || (command ? "center" /* center */ : getAlignByField(field))
            }, style, cellExternalProps.style);
            var classString = classNames(cellPrefix, field ? (_ref2 = {}, _defineProperty(_ref2, cellPrefix + '-dirty', !pristine && field.dirty), _defineProperty(_ref2, cellPrefix + '-required', !inlineEdit && field.required), _defineProperty(_ref2, cellPrefix + '-editable', !inlineEdit && this.hasEditor), _ref2) : void 0, className, cellExternalProps.className);
            return React.createElement(
                'td',
                _extends({}, cellExternalProps, { className: classString, style: omit(cellStyle, ['width', 'height']), 'data-index': getColumnKey(column) }),
                this.getInnerNode(cellPrefix, command)
            );
        }
    }, {
        key: 'showEditor',
        value: function showEditor(cell, lock) {
            var name = this.props.column.name;
            var cellEditor = this.cellEditor;

            if (name && cellEditor && !isRadio(cellEditor)) {
                if (!lock) {
                    var tableStore = this.context.tableStore;
                    var node = tableStore.node,
                        overflowX = tableStore.overflowX;

                    if (overflowX) {
                        var tableBodyWrap = cell.offsetParent;
                        if (tableBodyWrap) {
                            var leftLeafColumnsWidth = tableStore.leftLeafColumnsWidth,
                                rightLeafColumnsWidth = tableStore.rightLeafColumnsWidth;
                            var offsetLeft = cell.offsetLeft,
                                offsetWidth = cell.offsetWidth;
                            var scrollLeft = tableBodyWrap.scrollLeft;

                            var _tableBodyWrap$getBou = tableBodyWrap.getBoundingClientRect(),
                                width = _tableBodyWrap$getBou.width;

                            var leftSide = offsetLeft - leftLeafColumnsWidth;
                            var rightSide = offsetLeft + offsetWidth - width + rightLeafColumnsWidth + measureScrollbar();
                            var _scrollLeft = scrollLeft;
                            if (_scrollLeft < rightSide) {
                                _scrollLeft = rightSide;
                            }
                            if (_scrollLeft > leftSide) {
                                _scrollLeft = leftSide;
                            }
                            if (_scrollLeft !== scrollLeft) {
                                tableBodyWrap.scrollLeft = _scrollLeft;
                                node.handleBodyScrollLeft({
                                    target: tableBodyWrap,
                                    currentTarget: tableBodyWrap
                                });
                            }
                        }
                    }
                }
                this.context.tableStore.showEditor(name);
            }
        }
    }, {
        key: 'cellEditor',
        get: function get() {
            var _props7 = this.props,
                column = _props7.column,
                record = _props7.record;

            return getEditorByColumnAndRecord(column, record);
        }
    }, {
        key: 'cellEditorInCell',
        get: function get() {
            return isRadio(this.cellEditor);
        }
    }, {
        key: 'hasEditor',
        get: function get() {
            return !this.context.tableStore.pristine && this.cellEditor && !this.cellEditorInCell;
        }
    }]);

    return TableCell;
}(Component);
TableCell.displayName = 'TableCell';
TableCell.propTypes = {
    prefixCls: PropTypes.string,
    column: PropTypes.object.isRequired,
    record: PropTypes.instanceOf(Record).isRequired,
    indentSize: PropTypes.number.isRequired
};
TableCell.contextType = TableContext;
tslib_1.__decorate([computed], TableCell.prototype, "cellEditor", null);
tslib_1.__decorate([computed], TableCell.prototype, "cellEditorInCell", null);
tslib_1.__decorate([computed], TableCell.prototype, "hasEditor", null);
tslib_1.__decorate([autobind], TableCell.prototype, "handleEditorKeyDown", null);
tslib_1.__decorate([autobind], TableCell.prototype, "handleFocus", null);
tslib_1.__decorate([autobind], TableCell.prototype, "handleCommandEdit", null);
tslib_1.__decorate([autobind], TableCell.prototype, "handleCommandDelete", null);
tslib_1.__decorate([autobind], TableCell.prototype, "handleCommandSave", null);
tslib_1.__decorate([autobind], TableCell.prototype, "handleCommandCancel", null);
tslib_1.__decorate([autobind], TableCell.prototype, "renderCommand", null);
tslib_1.__decorate([autobind], TableCell.prototype, "renderEditor", null);
TableCell = tslib_1.__decorate([observer], TableCell);
export default TableCell;